#!/usr/local/bin/python

# GLTools PpenGL Basics Wrapping by Javi Agenjo '09 javi.agenjo@gmail.com

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from OpenGL.GL.ARB.framebuffer_object import *
from OpenGL.GL.EXT.framebuffer_object import *

from PIL import Image

import pygame
import Image
import sys,time

from ctypes import *
from math import *

def createAndCompileShader(type,source):
        shader=glCreateShader(type)
        glShaderSource(shader,source)
        glCompileShader(shader)

        # get "compile status" - glCompileShader will not fail with
        # an exception in case of syntax errors

        result=glGetShaderiv(shader,GL_COMPILE_STATUS)

        if (result!=1): # shader didn't compile
                raise Exception("Couldn't compile shader\nShader compilation Log:\n"+glGetShaderInfoLog(shader))
        return shader

class Shader:
    def __init__(self):
        print "Error: parameters needed"
        
    def __init__(self, vs_source, ps_source):
        
        vs_shader = createAndCompileShader(GL_VERTEX_SHADER,vs_source)
        ps_shader = createAndCompileShader(GL_FRAGMENT_SHADER,ps_source)

        self.program = glCreateProgram()
        glAttachShader(self.program,vs_shader)
        glAttachShader(self.program,ps_shader)
        glLinkProgram(self.program)

        self.enable()
        self.disable()
        self.last_slot = 0
        
    def setUniform1f(self,name,value):
        loc = glGetUniformLocation(self.program,name);
        if (loc == -1):
            print "Error: "+name+" var not found in shader"
        else:
            glUniform1f(loc,value);

    def setUniform2f(self,name,value):
        loc = glGetUniformLocation(self.program,name);
        if (loc == -1):
            print "Error: "+name+" var not found in shader"
        else:
            glUniform2f(loc,value[0],value[1]);


    def setUniformTexture(self,name,tex_id):
        loc = glGetUniformLocation(self.program,name);
        if (loc == -1):
            print "Error: "+name+" var not found in shader"
        else:
                glActiveTexture( GL_TEXTURE0 + self.last_slot )
                glBindTexture( GL_TEXTURE_2D, tex_id )
                glUniform1i(loc,self.last_slot)
                self.last_slot += 1

    def enable(self):
        try:
                glUseProgram(self.program)
                self.last_slot = 0
        except OpenGL.error.GLError:
                print glGetProgramInfoLog(self.program)
                raise

    def disable(self):
            glUseProgram(0)

# ******************************************************

class Texture:
    """Params: mipmaps (contains mipmaps), mag_filter, min_filter, wrap"""

    format_channels = { GL_RGB:[3],	GL_RGBA:[4],	GL_LUMINANCE:[1],	GL_DEPTH_COMPONENT:[1], GL_ALPHA16F_ARB:[1,GL_FLOAT],
                        GL_ALPHA32F_ARB:[1,GL_FLOAT],	GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:[4], GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:[4],
                        GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:[4],	GL_LUMINANCE32F_ARB:[1],	GL_LUMINANCE_ALPHA32F_ARB:[1,GL_FLOAT],
                        GL_RGB32F_ARB:[3,GL_FLOAT],	GL_RGBA16_EXT:[4], GL_RGBA16F_ARB:[4,GL_FLOAT], GL_RGBA32F_ARB:[4,GL_FLOAT] }

    def generic_init(self,parameters):
        #global data
        self.ready = False
        self.parameters = {}
        self.parameters['mipmaps'] = True
        self.parameters['mag_filter'] = GL_LINEAR
        self.parameters['min_filter'] = GL_LINEAR_MIPMAP_LINEAR        
        self.parameters['wrap'] = GL_REPEAT
        self.parameters.update(parameters)
        
    def __init__(self,parameters={}):
        self.generic_init(parameters)

    def load(self,filename):

        try:
                im = Image.open(filename)
        except IOError:
                print "File " + filename + " could not be opened"
                self.ready = False
                return False

        self.filename = filename
        self.size = im.size
        self.mode = im.mode
        self.channels = len(im.getbands())

        #data = im.tostring("raw", "RGBX", 0, -1)
        data = im.tostring()

        types = [GL_NONE, GL_LUMINANCE, GL_NONE, GL_RGB, GL_RGBA]
        self.depth = types[ self.channels ]
        #print "Channels " + str(self.channels)
                
        # generate texture
        if not hasattr(self,"texture_id"):
                self.texture_id=glGenTextures(1)

        self.updateParameters()
        glPixelStorei(GL_UNPACK_ALIGNMENT,1)
        if self.parameters['mipmaps'] == True:
                gluBuild2DMipmaps(GL_TEXTURE_2D, self.depth, self.size[0], self.size[1], GL_RGBA, GL_UNSIGNED_BYTE, data)
        else:
                glTexImage2D(GL_TEXTURE_2D, 0, self.depth, self.size[0], self.size[1], 0, GL_RGBA, GL_UNSIGNED_BYTE, data)
        self.ready = True
        #print len ( im.tostring() )
        return True

    def free(self):
        if hasattr(self,"texture_id"):
                glDeleteTextures(self.texture_id)
                delattr(self,texture_id)

    def bind(self):
        if not hasattr(self,"texture_id"):
                return
        glBindTexture( GL_TEXTURE_2D, self.texture_id )
        glEnable( GL_TEXTURE_2D )

    def unbind(self):
        glDisable( GL_TEXTURE_2D )        
                
    def render(self, pos=[0,0], size=[1,1]):
        if not hasattr(self,"texture_id"):
                return
            
        self.bind()
        glBegin(GL_QUADS)
        glColor3f(1,1,1)
        glTexCoord2f(0,0), glVertex3f(pos[0],pos[1],0)
        glTexCoord2f(1,0), glVertex3f(pos[0]+size[0],pos[1],0)
        glTexCoord2f(1,1), glVertex3f(pos[0]+size[0],pos[1]+size[1],0)
        glTexCoord2f(0,1), glVertex3f(pos[0],pos[1]+size[1],0)
        glEnd()
	
    def updateParameters(self):
        self.bind()
        glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,self.parameters['wrap'])
        glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,self.parameters['wrap'])
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, self.parameters['mag_filter'])
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, self.parameters['min_filter'])


class RenderTexture(Texture):
    def __init__(self,size,format=GL_RGB,parameters={},type=GL_UNSIGNED_BYTE):

        if not 'wrap' in parameters: parameters['wrap'] = GL_CLAMP
        Texture.__init__(self,parameters)
        
        if ( bool(glGenFramebuffers) == False):
                print "This graphics card do not support FBOs"
                self.ready = False
                return

        self.texture_id = glGenTextures( 1 )
        self.size = size
        self.internalFormat = format # Specifies the number of color components in the texture
        self.format = format # Specifies the format of the pixel data. GL_RGB or GL_RGBA
        self.format_type = type # Specifies the data type of the pixel data
        if format in Texture.format_channels:
                self.channels = Texture.format_channels[format][0]
        else:
                self.channels = 4

        glBindTexture( GL_TEXTURE_2D, self.texture_id )
        glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE )
        self.updateParameters()
        glTexImage2D(GL_TEXTURE_2D, 0, self.internalFormat, size[0],size[1], 0, GL_RGBA if self.channels == 4 else GL_RGB, self.format_type, None)

        if self.parameters['mipmaps']:
            glGenerateMipmap(GL_TEXTURE_2D)

        self.fbo = glGenFramebuffers(1)
        self.ready = True

    def enable(self):
        if not hasattr(self,"fbo"):
            return
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, self.fbo)
	# In case of errors or suspect behaviour, try this:
	#    print "Framebuffer Status:"
	#    print glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
	
        glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
        GL_TEXTURE_2D, self.texture_id, 0)
        glPushAttrib(GL_VIEWPORT_BIT) # save viewport
        glViewport(0, 0, self.size[0], self.size[1]) # set to fire effect dimensions

    def disable(self):
        if not hasattr(self,"fbo"):
            return
        glPopAttrib()
        glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0)
        if self.parameters['mipmaps']:
            glBindTexture( GL_TEXTURE_2D, self.texture_id )
            glGenerateMipmap(GL_TEXTURE_2D)

    def clear(self):
        self.enable()
        glClearColor(1.0, 0.0, 0.0, 0.0)
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        self.disable()


class Primitive:
    def __init__(self,type):

        self.cubelist = glGenLists(1);

        glNewList(self.cubelist,GL_COMPILE)

        if (type == "cube"):
            glBegin(GL_QUADS)
            glNormal3f(0,0,-1)
            glTexCoord2f(0,0), glVertex3f( -1, 1, -1)
            glTexCoord2f(1,0), glVertex3f( 1, 1, -1)
            glTexCoord2f(1,1), glVertex3f( 1, -1, -1)
            glTexCoord2f(0,1), glVertex3f( -1, -1, -1)
            
            glNormal3f(0,0,1)
            glTexCoord2f(1,1), glVertex3f( -1, -1, 1)
            glTexCoord2f(0,1), glVertex3f( 1, -1, 1)
            glTexCoord2f(0,0), glVertex3f( 1, 1, 1)
            glTexCoord2f(1,0), glVertex3f( -1, 1, 1)

            glNormal3f(0,-1,0)
            glTexCoord2f(0,1), glVertex3f( -1, -1, -1)
            glTexCoord2f(1,1), glVertex3f( 1, -1, -1)
            glTexCoord2f(1,0), glVertex3f( 1, -1, 1)
            glTexCoord2f(0,0), glVertex3f( -1, -1, 1)

            glNormal3f(0,1,0)
            glTexCoord2f(1,0), glVertex3f( -1, 1, 1)
            glTexCoord2f(0,0), glVertex3f( 1, 1, 1)
            glTexCoord2f(0,1), glVertex3f( 1, 1, -1)
            glTexCoord2f(1,1), glVertex3f( -1, 1, -1)

            glNormal3f(-1,0,0)
            glTexCoord2f(0,1), glVertex3f( -1, -1, 1)
            glTexCoord2f(0,0), glVertex3f( -1, 1, 1)
            glTexCoord2f(1,0), glVertex3f( -1, 1, -1)
            glTexCoord2f(1,1), glVertex3f( -1, -1, -1)

            glNormal3f(1,0,0)
            glTexCoord2f(0,1), glVertex3f( 1, -1, -1)
            glTexCoord2f(0,0), glVertex3f( 1, 1, -1)
            glTexCoord2f(1,0), glVertex3f( 1, 1, 1)
            glTexCoord2f(1,1), glVertex3f( 1, -1, 1)
            glEnd()
            
        glEndList()

    def render(self):
        glCallList(self.cubelist)

# *************************************
def enableRenderHUD(left=0,right=1,bottom=0,up=1):
    glPushAttrib( GL_ALL_ATTRIB_BITS )

    glDisable( GL_DEPTH_TEST )
    glDisable( GL_CULL_FACE )
    glDisable( GL_TEXTURE_2D )
    
    glMatrixMode( GL_MODELVIEW )
    glPushMatrix()
    glLoadIdentity()
    glMatrixMode( GL_PROJECTION )
    glPushMatrix()
    glLoadIdentity()
    gluOrtho2D(left,right,bottom,up)

def disableRenderHUD():
    glMatrixMode( GL_PROJECTION )
    glPopMatrix()
    glMatrixMode( GL_MODELVIEW )
    glPopMatrix()

    glPopAttrib()

def printGL(text,pos,color=[1.0,1.0,1.0]):
    glDisable( GL_TEXTURE_2D )
    glColor4f(color[0],color[1],color[2],1.0)
    glRasterPos2f(pos[0],pos[1]);
    for c in text:
        glutBitmapCharacter(GLUT_BITMAP_8_BY_13, ord(c) )

def distance(a,b):
    v = [ b[0]-a[0], b[1]-a[1], b[2]-a[2] ]
    return sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])

def jpg_file_write(name,number,size):
    data = glReadPixels(0,0,size[0],size[1],GL_RGBA, GL_UNSIGNED_BYTE)
    im = Image.frombuffer("RGBA", (size[0],size[1]), data, "raw", "RGBA", 0, 0)
    fnumber = "%05d" % number
    filename = name + fnumber + ".jpg"
    try:
            im.save(filename)
    except IOError:
            print "Error saving the screenshot in "+filename+" .Check that the folder exist." 
            return False
    print "Screenshot saved as " + filename
    return True
