#!/usr/local/bin/python

from OpenGL.GL import *
from OpenGL.GLU import *

from GLTools import *
from shaders import *

from Application import *
from math import * # trigonometry

WINDOW_SIZE = [512,512,False]
WINDOW_SIZE = [800,600,False]
#WINDOW_SIZE = [1440,900,True]
#WINDOW_SIZE = [1650,1050,True]

QUALITY = 1 # 2 is the max


class Brush:

    NORMAL=1
    INCREMENTAL=3
    RANDOM=2
    
    color = [1.0, 1.0, 1.0]
    size = [100,100]
    preassure = 0.1
    time_interval_in_ms = 20 #in secs
    angle = 0
    brush_texture_mode = INCREMENTAL
    brush_rotation_mode = INCREMENTAL
    brush_size_mode = NORMAL
    blend = True
    
    last_texture_used = 0
    last_draw_time = 0

    def __init__(self,textures):
        self.textures = []
        if len(textures) > 0:
            self.addTextures(textures)

    def addTextures(self,filename_list):

        for name in filename_list:
            t = Texture()
            t.load(name)
            self.textures.append(t)

    def incTexture(self):
        self.last_texture_used += 1
        if self.last_texture_used >= len(self.textures):
            self.last_texture_used = 0

    def draw(self, final = True):
        if final and self.time_interval_in_ms + self.last_draw_time > Application.instance.current_time:
            return #nothing to draw

        if self.brush_texture_mode == Brush.RANDOM:
            t = random.choice( self.textures )
        elif self.brush_texture_mode == Brush.INCREMENTAL:
            t = self.textures[ self.last_texture_used ]
        else:
             t = self.textures[0]
             
        t.bind()
        glColor4f(self.color[0],self.color[1],self.color[2],self.preassure if final else self.preassure * 2)

        if self.blend:
            glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
        else:
            glBlendFunc(GL_SRC_ALPHA,GL_ONE)
        glEnable(GL_BLEND)        

        glBegin( GL_QUADS )
        glTexCoord2f(0,0), glVertex3f( -self.size[0], -self.size[0], 0)
        glTexCoord2f(1,0), glVertex3f( +self.size[0], -self.size[0], 0)
        glTexCoord2f(1,1), glVertex3f( +self.size[0], +self.size[1], 0)
        glTexCoord2f(0,1), glVertex3f( -self.size[0], +self.size[1], 0)
        glEnd()

        if final:
            self.updateBrush()

    def updateBrush(self):
        self.last_draw_time = Application.instance.current_time

        if self.brush_texture_mode == Brush.INCREMENTAL:
            self.incTexture()

        if self.brush_rotation_mode == Brush.INCREMENTAL:
            self.angle += 5
        elif self.brush_rotation_mode == Brush.RANDOM:
            self.angle += random.randint(0,360)
        if self.brush_size_mode == Brush.RANDOM:
            self.size = [ random.randint(0,100),random.randint(0,100) ]
        

class Canvas:
    offset = [0,0]
    bgcolor = [0.0,0.0,0.0]
    quality = [[GL_RGB,GL_UNSIGNED_BYTE],[GL_RGBA16F_ARB,GL_UNSIGNED_SHORT],[GL_RGB32F_ARB,GL_FLOAT] ]
    def __init__(self,size):
        global QUALITY
        self.rt = RenderTexture(size,Canvas.quality[ QUALITY ][0],{}, Canvas.quality[ QUALITY ][1]) #GL_RGB32F_ARB,{},GL_FLOAT
        self.size = size
        self.clear()

        self.createUndo()

    def clear(self):
        enableRenderHUD(0,self.size[0], self.size[1], 0)
        self.rt.enable()        
        glClearColor(self.bgcolor[0], self.bgcolor[1], self.bgcolor[2], 1.0)
        glClear(GL_COLOR_BUFFER_BIT)
        self.rt.disable()
        disableRenderHUD()        

    def render(self, window_size):
        glMatrixMode( GL_MODELVIEW )
        #glPushMatrix()
        #glTranslate( self.offset[0], self.offset[1], 0.0)
        self.rt.render([0,0],window_size)
        #glPopMatrix()

    def paint(self,brush,pos,remove=False):

        self.rt.enable()
        enableRenderHUD(0,self.size[0], 0,self.size[1])

        self.renderBrush(brush,pos)

        disableRenderHUD()
        self.rt.disable()

    def renderBrush(self,brush,pos,final = True):

        glMatrixMode( GL_MODELVIEW )
        glPushMatrix()

        glTranslate(pos[0],pos[1],0)
        glRotate(brush.angle,0,0,1)

        brush.draw(final)

        glPopMatrix()

    def createUndo(self):
        self.undo_rt = RenderTexture(self.rt.size,self.rt.format,{},self.rt.format_type)
 
    def saveUndo(self):
        if not hasattr(self,"undo_rt"):
            return
        self.undo_rt.enable()
        
        enableRenderHUD(0,self.size[0], 0,self.size[1])
        glMatrixMode( GL_MODELVIEW )
        glPushMatrix()
        glTranslate( self.offset[0], self.offset[1], 0.0)
        self.rt.render([0,0],self.size)
        glPopMatrix()        
        self.undo_rt.disable()
        disableRenderHUD()

    def recoverUndo(self):
        if not hasattr(self,"undo_rt"):
            return
        self.rt,self.undo_rt = self.undo_rt,self.rt

    

class MyApp(Application):

    flags = {'cicling_colors':True,'drawing':False }
    brushes = []

    def init(self):
        glDisable( GL_CULL_FACE )        
        glDisable( GL_DEPTH_TEST )
        self.logo_tex = Texture()
        self.logo_tex.load("data/tmt-logo.png")

        self.canvas = Canvas(self.size)

        self.brushes.append( Brush(["data/brush-circle2.tga"] ) )
        self.brushes.append( Brush(["data/brush.png"] ) )
        self.brushes.append( Brush(["data/brush-circle.png"] ) )
        self.brushes.append( Brush(["data/star-1.png", "data/star-2.png", "data/star-3.png", "data/star-4.png", "data/star-5.png" ]) )
        self.brushes.append( Brush(["data/stencil-mrt.png", "data/stencil-obama.png", "data/stencil-house.png"] ) )
        print "Brushes loaded: " + str( len(self.brushes) )
        self.brush = self.brushes[0]

    def render(self):
        glClearColor(0.0, 0.0, 0.0, 1.0)
        glClear(GL_COLOR_BUFFER_BIT)

        enableRenderHUD(0,self.size[0], self.size[1], 0)

        self.canvas.render(self.size)
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
        glEnable(GL_BLEND)
        self.canvas.renderBrush(self.brush,self.mouse[:2],False)

        printGL("Preassure: " + "%3.3f" % self.brush.preassure,[10,self.size[1]-10],[1.0,1.0,1.0])
        printGL("Flow: " + "%3.3f" % (1000.0 / self.brush.time_interval_in_ms),[180,self.size[1]-10],[1.0,1.0,1.0])
        printGL("Texture: " + self.brush.textures[0].filename,[340,self.size[1]-10],[1.0,1.0,1.0])
        disableRenderHUD()

        self.renderFPSbar(self.elapsed_time)

    def update(self, time_in_ms):

        if MyApp.flags['drawing']:
            self.canvas.paint( self.brush, self.mouse[:2] )
        #print "-"*30

        if MyApp.flags['cicling_colors']:
            self.brush.color = [ sin(self.current_time * 0.001)*0.5+0.5, 0.3, cos(self.current_time * 0.001)*0.5]
        else:
            self.brush.color = [1.0] * 3
        self.brush.brush_rotation_mode = Brush.RANDOM if self.keys[pygame.K_LSHIFT] else Brush.INCREMENTAL
        self.brush.blend = True if self.keys[pygame.K_LCTRL] else False

        if self.keys[pygame.K_KP_MULTIPLY]:
            self.brush.time_interval_in_ms *= 1.1
        if self.keys[pygame.K_KP_DIVIDE]:
            self.brush.time_interval_in_ms *= 0.9
        if self.keys[pygame.K_KP_PLUS]:
            self.brush.preassure *= 1.1
        if self.keys[pygame.K_KP_MINUS]:
            self.brush.preassure *= 0.9

        #print "Stack: " + str( glGetIntegerv(GL_MODELVIEW_STACK_DEPTH) )

    def takeScreenshot(self):
        enableRenderHUD(0,self.size[0], self.size[1], 0)
        self.canvas.render(self.size)
        disableRenderHUD()
        jpg_file_write("screenshots/my_image",self.current_time,self.size)        

    def onMouseButtonPressed(self,button):
        if button == 1: #left
            self.canvas.saveUndo()
            MyApp.flags['drawing'] = True

        if self.keys[pygame.K_LSHIFT]:
            if button == 4: #wheelup
                self.brush.angle += 5
            elif button == 5: #wheeldown
                self.brush.angle -= 5
        else:
            if button == 4: #wheelup
                self.brush.size = [i*1.1 for i in self.brush.size]
            elif button == 5: #wheeldown
                self.brush.size = [i*0.9 for i in self.brush.size]

    def onMouseButtonReleased(self,button):
        if button == 1: #left
            MyApp.flags['drawing'] = False

    def onKeyPressed(self,key):
        Application.onKeyPressed(self,key)
        if key == pygame.K_s and self.keys[pygame.K_LCTRL]:
            self.takeScreenshot()
        if key == pygame.K_c:
            self.canvas.clear()
        if key == pygame.K_LALT:
            pass
        if key == pygame.K_KP_ENTER:
            
            self.brush = random.choice( self.brushes )
        if key == pygame.K_KP0:
            self.brush.incTexture()
        if key == pygame.K_KP_PERIOD:
            MyApp.flags['cicling_colors'] = not MyApp.flags['cicling_colors']
        if key == pygame.K_z and self.keys[pygame.K_LCTRL]:
            self.canvas.recoverUndo()
        if key == pygame.K_1: self.brush = self.brushes[0]
        if key == pygame.K_2 and len(self.brushes) > 1: self.brush = self.brushes[1]
        if key == pygame.K_3 and len(self.brushes) > 2: self.brush = self.brushes[2]
        if key == pygame.K_4 and len(self.brushes) > 3: self.brush = self.brushes[3]
        if key == pygame.K_5 and len(self.brushes) > 4: self.brush = self.brushes[4]

app = MyApp()
app.createWindow("My App", WINDOW_SIZE, WINDOW_SIZE[2] )
app.start()
    
