

light_vs="""
varying vec3 v;
varying vec3 N;
varying vec4 C;

void main(void)
{

v = (gl_ModelViewMatrix * gl_Vertex).xyz;
N = (gl_NormalMatrix * gl_Normal).xyz;
C = gl_Color;
gl_TexCoord[0] = gl_MultiTexCoord0;
gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;

}
""";

light_ps="""
varying vec3 N;
varying vec3 v;
varying vec4 C;
uniform float f;

void main(void)
{
vec3 L = gl_LightSource[0].position.xyz-v;
float Idiff = max(dot(normalize(L),N),0.0)*pow(length(L),-2.0);

gl_FragColor = C + f * vec4(1.0,1.0,1.0,1.0)*Idiff; // diffuse reflection
}
""";

texture_ps="""
uniform sampler2D texture;
varying vec4 C;
void main(void)
{
gl_FragColor = C * texture2D(texture,gl_TexCoord[0].st)
}
""";


basic_vs="""
void main(void)
{
    gl_TexCoord[0] = gl_MultiTexCoord0;
    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
}
"""

postfx_ps="""
uniform sampler2D texture;
uniform float off;

void main(void)
{
    vec4 final = vec4(0,0,0,1);
    float d = length(gl_TexCoord[0].st - vec2(0.5,0.5)) + off;
    final.x = texture2D(texture,gl_TexCoord[0].st - vec2(0.02 * d,0),d * 3).x;
    final.y = texture2D(texture,gl_TexCoord[0].st).y;
    final.z = texture2D(texture,gl_TexCoord[0].st + vec2(0.02 * d,0),d * 3).z;
    final *= abs( sin( gl_TexCoord[0].t * 600 ) );
    final += 0.5 * texture2D(texture,gl_TexCoord[0].st,3.0);
    
    gl_FragColor = final;
}
"""

conway_ps="""
uniform sampler2D texture;
uniform vec2 delta_cell;
uniform float cwt; //1.0 is a good value
uniform float pop_off;

vec3 readCell(float x, float y)
{
    vec3 result = vec3(0.0,0.0,0.0);
    vec4 pixel = texture2D(texture,gl_TexCoord[0].st + vec2(x*delta_cell.x,y*delta_cell.y));
    if (pixel.x > 0.5) result.x = 1.0;
    if (pixel.y > 0.5) result.y = 1.0;
    if (pixel.z > 0.5) result.z = 1.0;
    return result;
}

int cellResult(int isAlive,int cells_alive)
{
    if (isAlive == (0)) //dead
    {
        if (cells_alive == (3)) //reproduce
            return 1;
        return 0; //stays dead
    }
    
    //alive
    if (cells_alive == (2) || cells_alive == (3+pop_off) )
        return 1; //stay alive

    return 0; //dies
}

void main(void)
{
    vec3 alive = vec3(0.0,0.0,0.0);
    vec3 result = vec3(0.0,0.0,0.0);
    
    alive += readCell(-1,+1);
    alive += readCell(+0,+1);
    alive += readCell(+1,+1);
    
    alive += readCell(-1,0);
    alive += readCell(+1,0);

    alive += readCell(-1,-1);
    alive += readCell(+0,-1);
    alive += readCell(+1,-1);

    vec3 me = readCell(0,0);

    result.x = cellResult( me.x, alive.x );
    result.y = cellResult( me.y, alive.y );
    result.z = cellResult( me.z, alive.z );
    
    vec4 pixel = texture2D(texture,gl_TexCoord[0].st);
    //pixel += ( result ? vec4(cwt,cwt,cwt,1.0) : vec4(-cwt,-cwt,-cwt,0.0) );
    pixel.x += ( result.x ? cwt : -cwt );
    pixel.y += ( result.y ? cwt : -cwt );
    pixel.z += ( result.z ? cwt : -cwt );
    //pixel.y = pixel.x; pixel.z = pixel.x;
    
    pixel.a = 1.0;
    gl_FragColor = pixel;
}
"""
